@@ -36,15 +36,15 @@
 
  */
 
-#if !defined(I_IOBuffer_h)
+#pragma once
 #define I_IOBuffer_h
 
-#include "ts/ink_platform.h"
-#include "ts/ink_apidefs.h"
-#include "ts/Allocator.h"
-#include "ts/Ptr.h"
-#include "ts/ink_assert.h"
-#include "ts/ink_resource.h"
+#include "tscore/ink_platform.h"
+#include "tscore/ink_apidefs.h"
+#include "tscore/Allocator.h"
+#include "tscore/Ptr.h"
+#include "tscore/ink_assert.h"
+#include "tscore/ink_resource.h"
 
 struct MIOBufferAccessor;
 
@@ -228,7 +228,7 @@ public:
     should not use this object or reference after this call.
 
   */
-  virtual void free();
+  void free() override;
 
   int64_t _size_index;
 
@@ -269,10 +269,9 @@ public:
   {
   }
 
-private:
-  // declaration only
-  IOBufferData(const IOBufferData &);
-  IOBufferData &operator=(const IOBufferData &);
+  // noncopyable, declaration only
+  IOBufferData(const IOBufferData &) = delete;
+  IOBufferData &operator=(const IOBufferData &) = delete;
 };
 
 inkcoreapi extern ClassAllocator<IOBufferData> ioDataAllocator;
@@ -488,7 +487,7 @@ public:
     call.
 
   */
-  virtual void free();
+  void free() override;
 
   char *_start;
   char *_end;
@@ -520,9 +519,9 @@ public:
   */
   IOBufferBlock();
 
-private:
-  IOBufferBlock(const IOBufferBlock &);
-  IOBufferBlock &operator=(const IOBufferBlock &);
+  // noncopyable
+  IOBufferBlock(const IOBufferBlock &) = delete;
+  IOBufferBlock &operator=(const IOBufferBlock &) = delete;
 };
 
 extern inkcoreapi ClassAllocator<IOBufferBlock> ioBlockAllocator;
@@ -936,7 +935,7 @@ public:
   buf()
   {
     IOBufferBlock *b = first_write_block();
-    return b ? b->buf() : 0;
+    return b ? b->buf() : nullptr;
   }
 
   char *
@@ -1101,18 +1100,21 @@ public:
     if (_writer) {
       _writer->reset();
     }
-    for (int j = 0; j < MAX_MIOBUFFER_READERS; j++)
-      if (readers[j].allocated()) {
-        readers[j].reset();
+    for (auto &reader : readers) {
+      if (reader.allocated()) {
+        reader.reset();
       }
+    }
   }
 
   void
   init_readers()
   {
-    for (int j = 0; j < MAX_MIOBUFFER_READERS; j++)
-      if (readers[j].allocated() && !readers[j].block)
-        readers[j].block = _writer;
+    for (auto &reader : readers) {
+      if (reader.allocated() && !reader.block) {
+        reader.block = _writer;
+      }
+    }
   }
 
   void
@@ -1232,10 +1234,11 @@ struct MIOBufferAccessor {
   const char *name;
 #endif
 
-private:
-  MIOBufferAccessor(const MIOBufferAccessor &);
-  MIOBufferAccessor &operator=(const MIOBufferAccessor &);
+  // noncopyable
+  MIOBufferAccessor(const MIOBufferAccessor &) = delete;
+  MIOBufferAccessor &operator=(const MIOBufferAccessor &) = delete;
 
+private:
   MIOBuffer *mbuf;
   IOBufferReader *entry;
 };
@@ -1297,7 +1300,7 @@ extern IOBufferBlock *new_IOBufferBlock_internal(
 #ifdef TRACK_BUFFER_USER
   const char *loc
 #endif
-  );
+);
 
 extern IOBufferBlock *new_IOBufferBlock_internal(
 #ifdef TRACK_BUFFER_USER
@@ -1401,4 +1404,3 @@ extern IOBufferBlock *iobufferblock_clone(IOBufferBlock *b, int64_t offset, int6
 
 */
 extern IOBufferBlock *iobufferblock_skip(IOBufferBlock *b, int64_t *poffset, int64_t *plen, int64_t write);
-#endif
