@@ -21,14 +21,13 @@
   limitations under the License.
  */
 
-#ifndef _I_HostDBProcessor_h_
-#define _I_HostDBProcessor_h_
+#pragma once
 
-#include "ts/HashFNV.h"
-#include "ts/ink_time.h"
-#include "ts/INK_MD5.h"
-#include "ts/ink_align.h"
-#include "ts/ink_resolver.h"
+#include "tscore/HashFNV.h"
+#include "tscore/ink_time.h"
+#include "tscore/CryptoHash.h"
+#include "tscore/ink_align.h"
+#include "tscore/ink_resolver.h"
 #include "I_EventSystem.h"
 #include "SRV.h"
 #include "P_RefCountCache.h"
@@ -42,9 +41,6 @@
 #define EVENT_SRV_IP_REMOVED (SRV_EVENT_EVENTS_START + 1)
 #define EVENT_SRV_GET_RESPONSE (SRV_EVENT_EVENTS_START + 2)
 
-// TODO: make configurable
-#define HOST_DB_MAX_ROUND_ROBIN_INFO 16
-
 //
 // Data
 //
@@ -64,6 +60,7 @@ extern unsigned int hostdb_ip_stale_interval;
 extern unsigned int hostdb_ip_timeout_interval;
 extern unsigned int hostdb_ip_fail_timeout_interval;
 extern unsigned int hostdb_serve_stale_but_revalidate;
+extern unsigned int hostdb_round_robin_max_count;
 
 static inline unsigned int
 makeHostHash(const char *string)
@@ -159,7 +156,7 @@ struct HostDBInfo : public RefCountObj {
   }
 
   void
-  free()
+  free() override
   {
     Debug("hostdb", "freeing %d bytes at [%p]", (1 << (7 + iobuffer_index)), this);
     ioBufAllocator[iobuffer_index].free_void((void *)(this));
@@ -313,12 +310,13 @@ struct HostDBInfo : public RefCountObj {
   void
   set_failed()
   {
-    if (is_srv)
+    if (is_srv) {
       data.srv.srv_offset = 0;
-    else if (reverse_dns)
+    } else if (reverse_dns) {
       data.hostname_offset = 0;
-    else
+    } else {
       ats_ip_invalidate(ip());
+    }
   }
 
   int iobuffer_index;
@@ -477,7 +475,7 @@ struct HostDBProcessor : public Processor {
   void
   setbyaddr_appinfo(sockaddr const *addr, HostDBApplicationInfo *app)
   {
-    this->setby(0, 0, addr, app);
+    this->setby(nullptr, 0, addr, app);
   }
 
   void
@@ -485,7 +483,7 @@ struct HostDBProcessor : public Processor {
   {
     sockaddr_in addr;
     ats_ip4_set(&addr, ip);
-    this->setby(0, 0, ats_ip_sa_cast(&addr), app);
+    this->setby(nullptr, 0, ats_ip_sa_cast(&addr), app);
   }
 
   /** Configuration. */
@@ -496,7 +494,7 @@ struct HostDBProcessor : public Processor {
   /* hostdb does not use any dedicated event threads
    * currently. Dont pass any value to start
    */
-  int start(int no_of_additional_event_threads = 0, size_t stacksize = DEFAULT_STACKSIZE);
+  int start(int no_of_additional_event_threads = 0, size_t stacksize = DEFAULT_STACKSIZE) override;
 
   // Private
   HostDBCache *cache();
@@ -515,7 +513,7 @@ public:
              int len,                   ///< Length of hostname.
              sockaddr const *aip,       ///< Address and/or port.
              HostDBApplicationInfo *app ///< I don't know.
-             );
+  );
 
   void setby_srv(const char *hostname, int len, const char *target, HostDBApplicationInfo *app);
 };
@@ -525,5 +523,3 @@ void run_HostDBTest();
 extern inkcoreapi HostDBProcessor hostDBProcessor;
 
 void ink_hostdb_init(ModuleVersion version);
-
-#endif
