@@ -19,11 +19,10 @@
   limitations under the License.
  */
 
-#ifndef IOCORE_NET_P_SSLCLIENTUTILS_H_
-#define IOCORE_NET_P_SSLCLIENTUTILS_H_
+#pragma once
 
-#include "ts/ink_config.h"
-#include "ts/Diags.h"
+#include "tscore/ink_config.h"
+#include "tscore/Diags.h"
 #include "P_SSLUtils.h"
 #include "P_SSLConfig.h"
 
@@ -38,5 +37,3 @@
 SSL_CTX *SSLInitClientContext(const struct SSLConfigParams *param);
 
 int verify_callback(int preverify_ok, X509_STORE_CTX *ctx);
-
-#endif /* IOCORE_NET_P_SSLCLIENTUTILS_H_ */
