@@ -21,13 +21,13 @@
   limitations under the License.
  */
 
-#include "ts/ink_platform.h"
-#include "ts/ink_memory.h"
+#include "tscore/ink_platform.h"
+#include "tscore/ink_memory.h"
 
-#include "ts/TextBuffer.h"
-#include "ts/Tokenizer.h"
-#include "ts/ink_defs.h"
-#include "ts/ink_string.h"
+#include "tscore/TextBuffer.h"
+#include "tscore/Tokenizer.h"
+#include "tscore/ink_defs.h"
+#include "tscore/ink_string.h"
 
 #include "P_RecFile.h"
 #include "P_RecUtils.h"
@@ -261,21 +261,20 @@ recv_message_cb(RecMessage *msg, RecMessageT msg_type, void * /* cookie */)
 //-------------------------------------------------------------------------
 // RecRegisterStatXXX
 //-------------------------------------------------------------------------
-#define REC_REGISTER_STAT_XXX(A, B)                                                                                             \
-  ink_assert((rec_type == RECT_NODE) || (rec_type == RECT_CLUSTER) || (rec_type == RECT_PROCESS) || (rec_type == RECT_LOCAL) || \
-             (rec_type == RECT_PLUGIN));                                                                                        \
-  RecRecord *r;                                                                                                                 \
-  RecData my_data_default;                                                                                                      \
-  my_data_default.A = data_default;                                                                                             \
-  if ((r = RecRegisterStat(rec_type, name, B, my_data_default, persist_type)) != nullptr) {                                     \
-    if (i_am_the_record_owner(r->rec_type)) {                                                                                   \
-      r->sync_required = r->sync_required | REC_PEER_SYNC_REQUIRED;                                                             \
-    } else {                                                                                                                    \
-      send_register_message(r);                                                                                                 \
-    }                                                                                                                           \
-    return REC_ERR_OKAY;                                                                                                        \
-  } else {                                                                                                                      \
-    return REC_ERR_FAIL;                                                                                                        \
+#define REC_REGISTER_STAT_XXX(A, B)                                                                                           \
+  ink_assert((rec_type == RECT_NODE) || (rec_type == RECT_PROCESS) || (rec_type == RECT_LOCAL) || (rec_type == RECT_PLUGIN)); \
+  RecRecord *r;                                                                                                               \
+  RecData my_data_default;                                                                                                    \
+  my_data_default.A = data_default;                                                                                           \
+  if ((r = RecRegisterStat(rec_type, name, B, my_data_default, persist_type)) != nullptr) {                                   \
+    if (i_am_the_record_owner(r->rec_type)) {                                                                                 \
+      r->sync_required = r->sync_required | REC_PEER_SYNC_REQUIRED;                                                           \
+    } else {                                                                                                                  \
+      send_register_message(r);                                                                                               \
+    }                                                                                                                         \
+    return REC_ERR_OKAY;                                                                                                      \
+  } else {                                                                                                                    \
+    return REC_ERR_FAIL;                                                                                                      \
   }
 
 RecErrT
@@ -376,22 +375,25 @@ RecSetRecord(RecT rec_type, const char *name, RecDataT data_type, RecData *data,
       if ((data_type != RECD_NULL) && (r1->data_type != data_type)) {
         err = REC_ERR_FAIL;
       } else {
+        bool rec_updated_p = false;
         if (data_type == RECD_NULL) {
           // If the caller didn't know the data type, they gave us a string
           // and we should convert based on the record's data type.
           ink_release_assert(data->rec_string != nullptr);
-          RecDataSetFromString(r1->data_type, &(r1->data), data->rec_string);
+          rec_updated_p = RecDataSetFromString(r1->data_type, &(r1->data), data->rec_string);
         } else {
-          RecDataSet(data_type, &(r1->data), data);
+          rec_updated_p = RecDataSet(data_type, &(r1->data), data);
         }
 
-        r1->sync_required = REC_SYNC_REQUIRED;
-        if (inc_version) {
-          r1->sync_required |= REC_INC_CONFIG_VERSION;
-        }
+        if (rec_updated_p) {
+          r1->sync_required = REC_SYNC_REQUIRED;
+          if (inc_version) {
+            r1->sync_required |= REC_INC_CONFIG_VERSION;
+          }
 
-        if (REC_TYPE_IS_CONFIG(r1->rec_type)) {
-          r1->config_meta.update_required = REC_UPDATE_REQUIRED;
+          if (REC_TYPE_IS_CONFIG(r1->rec_type)) {
+            r1->config_meta.update_required = REC_UPDATE_REQUIRED;
+          }
         }
 
         if (REC_TYPE_IS_STAT(r1->rec_type) && (data_raw != nullptr)) {
@@ -627,7 +629,7 @@ RecReadConfigFile(bool inc_version)
 // RecSyncConfigFile
 //-------------------------------------------------------------------------
 RecErrT
-RecSyncConfigToTB(textBuffer *tb, bool *inc_version)
+RecSyncConfigToTB(TextBuffer *tb, bool *inc_version)
 {
   RecErrT err = REC_ERR_FAIL;
 
@@ -679,7 +681,7 @@ RecSyncConfigToTB(textBuffer *tb, bool *inc_version)
     if (sync_to_disk) {
       char b[1024];
 
-      // okay, we're going to write into our textBuffer
+      // okay, we're going to write into our TextBuffer
       err = REC_ERR_OKAY;
       tb->reUse();
 
@@ -705,9 +707,6 @@ RecSyncConfigToTB(textBuffer *tb, bool *inc_version)
             case RECT_NODE:
               tb->copyFrom("NODE ", 5);
               break;
-            case RECT_CLUSTER:
-              tb->copyFrom("CLUSTER ", 8);
-              break;
             case RECT_LOCAL:
               tb->copyFrom("LOCAL ", 6);
               break;
@@ -896,7 +895,7 @@ RecSetSyncRequired(char *name, bool lock)
   if (ink_hash_table_lookup(g_records_ht, name, (void **)&r1)) {
     if (i_am_the_record_owner(r1->rec_type)) {
       rec_mutex_acquire(&(r1->lock));
-      r1->sync_required = REC_SYNC_REQUIRED;
+      r1->sync_required = REC_PEER_SYNC_REQUIRED;
       if (REC_TYPE_IS_CONFIG(r1->rec_type)) {
         r1->config_meta.update_required = REC_UPDATE_REQUIRED;
       }
@@ -931,7 +930,7 @@ RecSetSyncRequired(char *name, bool lock)
 }
 
 RecErrT
-RecWriteConfigFile(textBuffer *tb)
+RecWriteConfigFile(TextBuffer *tb)
 {
 #define TMP_FILENAME_EXT_STR ".tmp"
 #define TMP_FILENAME_EXT_LEN (sizeof(TMP_FILENAME_EXT_STR) - 1)
