@@ -29,8 +29,8 @@
  *
  ***************************************************************************/
 
-#include "ts/ink_platform.h"
-#include "ts/ink_sock.h"
+#include "tscore/ink_platform.h"
+#include "tscore/ink_sock.h"
 #include "LocalManager.h"
 #include "MgmtSocket.h"
 #include "MgmtMarshall.h"
@@ -122,13 +122,7 @@ remove_event_client(EventClientT *client, InkHashTable *table)
 TSMgmtError
 init_mgmt_events()
 {
-  int ret;
-
-  ret = ink_mutex_init(&mgmt_events_lock, "mgmt_event_notice");
-
-  if (ret) {
-    return TS_ERR_SYS_CALL;
-  }
+  ink_mutex_init(&mgmt_events_lock);
 
   // initialize queue
   mgmt_events = create_queue();
@@ -525,8 +519,6 @@ done:
 using event_message_handler = TSMgmtError (*)(EventClientT *, void *, size_t);
 
 static const event_message_handler handlers[] = {
-  nullptr,                     // FILE_READ
-  nullptr,                     // FILE_WRITE
   nullptr,                     // RECORD_SET
   nullptr,                     // RECORD_GET
   nullptr,                     // PROXY_STATE_GET
@@ -540,16 +532,13 @@ static const event_message_handler handlers[] = {
   handle_event_reg_callback,   // EVENT_REG_CALLBACK
   handle_event_unreg_callback, // EVENT_UNREG_CALLBACK
   nullptr,                     // EVENT_NOTIFY
-  nullptr,                     // SNAPSHOT_TAKE
-  nullptr,                     // SNAPSHOT_RESTORE
-  nullptr,                     // SNAPSHOT_REMOVE
-  nullptr,                     // SNAPSHOT_GET_MLT
   nullptr,                     // DIAGS
   nullptr,                     // STATS_RESET_NODE
-  nullptr,                     // STATS_RESET_CLUSTER
   nullptr,                     // STORAGE_DEVICE_CMD_OFFLINE
   nullptr,                     // RECORD_MATCH_GET
   nullptr,                     // LIFECYCLE_MESSAGE
+  nullptr,                     // HOST_STATUS_UP
+  nullptr,                     // HOST_STATUS_DOWN
 };
 
 static TSMgmtError
