@@ -21,7 +21,7 @@
 #include "ts/ts.h"
 #include "ts/remap.h"
 
-#include "ts/ink_atomic.h"
+#include "tscore/ink_atomic.h"
 
 #include "parser.h"
 #include "ruleset.h"
@@ -46,7 +46,10 @@ initGeoIP()
     if (!gGeoIP[dbs[i]] && GeoIP_db_avail(dbs[i])) {
       // GEOIP_STANDARD seems to break threaded apps...
       gGeoIP[dbs[i]] = GeoIP_open_type(dbs[i], GEOIP_MMAP_CACHE);
-      TSDebug(PLUGIN_NAME, "initialized GeoIP-DB[%d] %s", dbs[i], GeoIP_database_info(gGeoIP[dbs[i]]));
+
+      char *db_info = GeoIP_database_info(gGeoIP[dbs[i]]);
+      TSDebug(PLUGIN_NAME, "initialized GeoIP-DB[%d] %s", dbs[i], db_info);
+      free(db_info);
     }
   }
 }
@@ -67,8 +70,9 @@ static int cont_rewrite_headers(TSCont, TSEvent, void *);
 class RulesConfig
 {
 public:
-  RulesConfig() : _ref_count(0)
+  RulesConfig()
   {
+    TSDebug(PLUGIN_NAME_DBG, "RulesConfig CTOR");
     memset(_rules, 0, sizeof(_rules));
     memset(_resids, 0, sizeof(_resids));
 
@@ -76,6 +80,15 @@ public:
     TSContDataSet(_cont, static_cast<void *>(this));
   }
 
+  ~RulesConfig()
+  {
+    TSDebug(PLUGIN_NAME_DBG, "RulesConfig DTOR");
+    for (int i = TS_HTTP_READ_REQUEST_HDR_HOOK; i <= TS_HTTP_LAST_HOOK; ++i) {
+      delete _rules[i];
+    }
+    TSContDestroy(_cont);
+  }
+
   TSCont
   continuation() const
   {
@@ -95,32 +108,10 @@ public:
 
   bool parse_config(const std::string &fname, TSHttpHookID default_hook);
 
-  void
-  hold()
-  {
-    ink_atomic_increment(&_ref_count, 1);
-  }
-  void
-  release()
-  {
-    if (1 >= ink_atomic_decrement(&_ref_count, 1)) {
-      delete this;
-    }
-  }
-
 private:
-  ~RulesConfig()
-  {
-    for (int i = TS_HTTP_READ_REQUEST_HDR_HOOK; i < TS_HTTP_LAST_HOOK; ++i) {
-      delete _rules[i];
-    }
-    TSContDestroy(_cont);
-  }
-
   bool add_rule(RuleSet *rule);
 
   TSCont _cont;
-  volatile int _ref_count;
   RuleSet *_rules[TS_HTTP_LAST_HOOK + 1];
   ResourceIDs _resids[TS_HTTP_LAST_HOOK + 1];
 };
@@ -281,9 +272,6 @@ cont_rewrite_headers(TSCont contp, TSEvent event, void *edata)
   case TS_EVENT_HTTP_SEND_RESPONSE_HDR:
     hook = TS_HTTP_SEND_RESPONSE_HDR_HOOK;
     break;
-  case TS_EVENT_HTTP_TXN_CLOSE:
-    conf->release();
-    break;
   default:
     TSError("[%s] unknown event for this plugin", PLUGIN_NAME);
     TSDebug(PLUGIN_NAME, "unknown event for this plugin");
@@ -327,7 +315,7 @@ TSPluginInit(int argc, const char *argv[])
   info.support_email = (char *)"dev@trafficserver.apache.org";
 
   if (TS_SUCCESS != TSPluginRegister(&info)) {
-    TSError("[%s] plugin registration failed.", PLUGIN_NAME);
+    TSError("[%s] plugin registration failed", PLUGIN_NAME);
   }
 
   // Parse the global config file(s). All rules are just appended
@@ -336,7 +324,6 @@ TSPluginInit(int argc, const char *argv[])
   bool got_config   = false;
 
   initGeoIP();
-  conf->hold();
 
   for (int i = 1; i < argc; ++i) {
     // Parse the config file(s). Note that multiple config files are
@@ -363,7 +350,7 @@ TSPluginInit(int argc, const char *argv[])
   } else {
     // Didn't get anything, nuke it.
     TSError("[%s] failed to parse any configuration file", PLUGIN_NAME);
-    conf->release();
+    delete conf;
   }
 }
 
@@ -384,7 +371,7 @@ TSRemapInit(TSRemapInterface *api_info, char *errbuf, int errbuf_size)
   }
 
   if (api_info->tsremap_version < TSREMAP_VERSION) {
-    snprintf(errbuf, errbuf_size - 1, "[TSRemapInit] - Incorrect API version %ld.%ld", api_info->tsremap_version >> 16,
+    snprintf(errbuf, errbuf_size, "[TSRemapInit] - Incorrect API version %ld.%ld", api_info->tsremap_version >> 16,
              (api_info->tsremap_version & 0xffff));
     return TS_ERROR;
   }
@@ -407,12 +394,11 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char * /* errbuf ATS_UNUSE
 
   RulesConfig *conf = new RulesConfig;
 
-  conf->hold();
-
   for (int i = 2; i < argc; ++i) {
     TSDebug(PLUGIN_NAME, "Loading remap configuration file %s", argv[i]);
     if (!conf->parse_config(argv[i], TS_REMAP_PSEUDO_HOOK)) {
       TSError("[%s] Unable to create remap instance", PLUGIN_NAME);
+      delete conf;
       return TS_ERROR;
     } else {
       TSDebug(PLUGIN_NAME, "Succesfully loaded remap config file %s", argv[i]);
@@ -436,7 +422,7 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char * /* errbuf ATS_UNUSE
 void
 TSRemapDeleteInstance(void *ih)
 {
-  static_cast<RulesConfig *>(ih)->release();
+  delete static_cast<RulesConfig *>(ih);
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -445,8 +431,6 @@ TSRemapDeleteInstance(void *ih)
 TSRemapStatus
 TSRemapDoRemap(void *ih, TSHttpTxn rh, TSRemapRequestInfo *rri)
 {
-  bool hooked_p = false;
-
   // Make sure things are properly setup (this should never happen)
   if (nullptr == ih) {
     TSDebug(PLUGIN_NAME, "No Rules configured, falling back to default");
@@ -459,18 +443,11 @@ TSRemapDoRemap(void *ih, TSHttpTxn rh, TSRemapRequestInfo *rri)
   // Go through all hooks we support, and setup the txn hook(s) as necessary
   for (int i = TS_HTTP_READ_REQUEST_HDR_HOOK; i < TS_HTTP_LAST_HOOK; ++i) {
     if (conf->rule(i)) {
-      hooked_p = true;
       TSHttpTxnHookAdd(rh, static_cast<TSHttpHookID>(i), conf->continuation());
       TSDebug(PLUGIN_NAME, "Added remapped TXN hook=%s", TSHttpHookNameLookup((TSHttpHookID)i));
     }
   }
 
-  // Two assumptions - configuration never uses this hook nor uses TS_HTTP_SSN_CLOSE_HOOK.
-  if (hooked_p) {
-    conf->hold();                                                       // mark as in use.
-    TSHttpTxnHookAdd(rh, TS_HTTP_TXN_CLOSE_HOOK, conf->continuation()); // clean up after.
-  }
-
   // Now handle the remap specific rules for the "remap hook" (which is not a real hook).
   // This is sufficiently differen than the normal cont_rewrite_headers() callback, and
   // we can't (shouldn't) schedule this as a TXN hook.
