@@ -36,13 +36,24 @@ class RuleSet
 {
 public:
   RuleSet()
-    : next(NULL),
-      _cond(NULL),
-      _oper(NULL),
+    : next(nullptr),
+      _cond(nullptr),
+      _oper(nullptr),
       _hook(TS_HTTP_READ_RESPONSE_HDR_HOOK),
       _ids(RSRC_NONE),
       _opermods(OPER_NONE),
-      _last(false){};
+      _last(false)
+  {
+    TSDebug(PLUGIN_NAME_DBG, "RuleSet CTOR");
+  }
+
+  ~RuleSet()
+  {
+    TSDebug(PLUGIN_NAME_DBG, "RulesSet DTOR");
+    delete next;
+    delete _cond;
+    delete _oper;
+  }
 
   // No reason to inline these
   void append(RuleSet *rule);
@@ -53,13 +64,13 @@ public:
   bool
   has_operator() const
   {
-    return NULL != _oper;
+    return nullptr != _oper;
   }
 
   bool
   has_condition() const
   {
-    return NULL != _cond;
+    return nullptr != _cond;
   }
 
   void
@@ -83,7 +94,7 @@ public:
   bool
   eval(const Resources &res) const
   {
-    if (NULL == _cond) {
+    if (nullptr == _cond) {
       return true;
     } else {
       return _cond->do_eval(res);
