@@ -22,15 +22,15 @@
  */
 
 #include <cstdio>
-#include "ts/ink_platform.h"
-#include "ts/ink_file.h"
-#include "ts/ParseRules.h"
-#include "I_RecCore.h"
-#include "ts/I_Layout.h"
+#include "tscore/ink_platform.h"
+#include "tscore/ink_file.h"
+#include "tscore/ParseRules.h"
+#include "records/I_RecCore.h"
+#include "tscore/I_Layout.h"
 #include "InkAPIInternal.h"
 #include "Main.h"
 #include "Plugin.h"
-#include "ts/ink_cap.h"
+#include "tscore/ink_cap.h"
 
 #define MAX_PLUGIN_ARGS 64
 
@@ -126,7 +126,7 @@ plugin_load(int argc, char *argv[], bool validateOnly)
       return false; // this line won't get called since Fatal brings down ATS
     }
 
-#if defined(freebsd) || defined(darwin)
+#if (!defined(kfreebsd) && defined(freebsd)) || defined(darwin)
     optreset = 1;
 #endif
 #if defined(__GLIBC__)
@@ -299,8 +299,12 @@ plugin_init(bool validateOnly)
         argv[i] = vars[i];
       }
     }
-    argv[argc] = nullptr;
 
+    if (argc < MAX_PLUGIN_ARGS) {
+      argv[argc] = nullptr;
+    } else {
+      argv[MAX_PLUGIN_ARGS - 1] = nullptr;
+    }
     retVal = plugin_load(argc, argv, validateOnly);
 
     for (i = 0; i < argc; i++) {
