@@ -23,13 +23,13 @@
 
 #include <cassert>
 #include <new>
-#include "ts/ink_platform.h"
-#include "ts/ink_memory.h"
-#include "ts/TsBuffer.h"
+#include "tscore/ink_platform.h"
+#include "tscore/ink_memory.h"
+#include "tscore/TsBuffer.h"
 #include "URL.h"
 #include "MIME.h"
 #include "HTTP.h"
-#include "ts/Diags.h"
+#include "tscore/Diags.h"
 
 const char *URL_SCHEME_FILE;
 const char *URL_SCHEME_FTP;
@@ -94,8 +94,8 @@ int URL_LEN_MMS;
 int URL_LEN_MMSU;
 int URL_LEN_MMST;
 
-// Whether we should implement url_MD5_get() using url_MD5_get_fast(). Note that
-// url_MD5_get_fast() does NOT produce the same result as url_MD5_get_general().
+// Whether we should implement url_CryptoHash_get() using url_CryptoHash_get_fast(). Note that
+// url_CryptoHash_get_fast() does NOT produce the same result as url_CryptoHash_get_general().
 static int url_hash_method = 0;
 
 // test to see if a character is a valid character for a host in a URI according to
@@ -109,35 +109,13 @@ is_host_char(char c)
 
 // Checks if `addr` is a valid FQDN string
 bool
-validate_host_name(ts::ConstBuffer addr)
+validate_host_name(std::string_view addr)
 {
-  while (addr) {
-    if (!(is_host_char(*addr))) {
-      return false;
-    }
-    ++addr;
-  }
-  return true;
+  return std::all_of(addr.begin(), addr.end(), &is_host_char);
 }
 
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
-URLHashContext::HashType URLHashContext::Setting = URLHashContext::MD5;
-
-URLHashContext::URLHashContext()
-{
-  switch (Setting) {
-  case UNSPECIFIED:
-  case MD5:
-    new (_obj) MD5Context;
-    break;
-  case MMH:
-    new (_obj) MMHContext;
-    break;
-  default:
-    ink_assert("Invalid global URL hash context");
-  };
-}
 
 void
 url_init()
@@ -216,9 +194,6 @@ url_init()
     URL_LEN_MMS      = hdrtoken_wks_to_length(URL_SCHEME_MMS);
     URL_LEN_MMSU     = hdrtoken_wks_to_length(URL_SCHEME_MMSU);
     URL_LEN_MMST     = hdrtoken_wks_to_length(URL_SCHEME_MMST);
-
-    ink_assert(URLHashContext::OBJ_SIZE >= sizeof(MD5Context));
-    ink_assert(URLHashContext::OBJ_SIZE >= sizeof(MMHContext));
   }
 }
 
@@ -1337,7 +1312,7 @@ url_parse_internet(HdrHeap *heap, URLImpl *url, const char **start, char const *
     }
   }
   if (host._size) {
-    if (validate_host_name(host)) {
+    if (validate_host_name(std::string_view(host._ptr, host._size))) {
       url_host_set(heap, url, host._ptr, host._size, copy_strings_p);
     } else {
       return PARSE_RESULT_ERROR;
@@ -1658,11 +1633,11 @@ memcpy_tolower(char *d, const char *s, int n)
 
 #define BUFSIZE 512
 
-// fast path for MD5, HTTP, no user/password/params/query,
+// fast path for CryptoHash, HTTP, no user/password/params/query,
 // no buffer overflow, no unescaping needed
 
 static inline void
-url_MD5_get_fast(const URLImpl *url, CryptoContext &ctx, CryptoHash *hash, cache_generation_t generation)
+url_CryptoHash_get_fast(const URLImpl *url, CryptoContext &ctx, CryptoHash *hash, cache_generation_t generation)
 {
   char buffer[BUFSIZE];
   char *p;
@@ -1697,11 +1672,11 @@ url_MD5_get_fast(const URLImpl *url, CryptoContext &ctx, CryptoHash *hash, cache
     ctx.update(&generation, sizeof(generation));
   }
 
-  ctx.finalize(hash);
+  ctx.finalize(*hash);
 }
 
 static inline void
-url_MD5_get_general(const URLImpl *url, CryptoContext &ctx, CryptoHash &hash, cache_generation_t generation)
+url_CryptoHash_get_general(const URLImpl *url, CryptoContext &ctx, CryptoHash &hash, cache_generation_t generation)
 {
   char buffer[BUFSIZE];
   char *p, *e;
@@ -1765,33 +1740,35 @@ url_MD5_get_general(const URLImpl *url, CryptoContext &ctx, CryptoHash &hash, ca
   if (p != buffer) {
     ctx.update(buffer, p - buffer);
   }
-
-  port = url_canonicalize_port(url->m_url_type, url->m_port);
+  int buffer_len = static_cast<int>(p - buffer);
+  port           = url_canonicalize_port(url->m_url_type, url->m_port);
 
   ctx.update(&port, sizeof(port));
   if (generation != -1) {
     ctx.update(&generation, sizeof(generation));
+    Debug("url_cachekey", "Final url string for cache hash key %.*s%d%d", buffer_len, buffer, port, static_cast<int>(generation));
+  } else {
+    Debug("url_cachekey", "Final url string for cache hash key %.*s%d", buffer_len, buffer, port);
   }
-
   ctx.finalize(hash);
 }
 
 void
-url_MD5_get(const URLImpl *url, CryptoHash *hash, cache_generation_t generation)
+url_CryptoHash_get(const URLImpl *url, CryptoHash *hash, cache_generation_t generation)
 {
   URLHashContext ctx;
   if ((url_hash_method != 0) && (url->m_url_type == URL_TYPE_HTTP) &&
       ((url->m_len_user + url->m_len_password + url->m_len_params + url->m_len_query) == 0) &&
       (3 + 1 + 1 + 1 + 1 + 1 + 2 + url->m_len_scheme + url->m_len_host + url->m_len_path < BUFSIZE) &&
       (memchr(url->m_ptr_host, '%', url->m_len_host) == nullptr) && (memchr(url->m_ptr_path, '%', url->m_len_path) == nullptr)) {
-    url_MD5_get_fast(url, ctx, hash, generation);
+    url_CryptoHash_get_fast(url, ctx, hash, generation);
 #ifdef DEBUG
-    CryptoHash md5_general;
-    url_MD5_get_general(url, ctx, md5_general, generation);
-    ink_assert(*hash == md5_general);
+    CryptoHash hash_general;
+    url_CryptoHash_get_general(url, ctx, hash_general, generation);
+    ink_assert(*hash == hash_general);
 #endif
   } else {
-    url_MD5_get_general(url, ctx, *hash, generation);
+    url_CryptoHash_get_general(url, ctx, *hash, generation);
   }
 }
 
@@ -1801,9 +1778,9 @@ url_MD5_get(const URLImpl *url, CryptoHash *hash, cache_generation_t generation)
   -------------------------------------------------------------------------*/
 
 void
-url_host_MD5_get(URLImpl *url, INK_MD5 *md5)
+url_host_CryptoHash_get(URLImpl *url, CryptoHash *hash)
 {
-  MD5Context ctx;
+  CryptoContext ctx;
 
   if (url->m_ptr_scheme) {
     ctx.update(url->m_ptr_scheme, url->m_len_scheme);
@@ -1818,17 +1795,17 @@ url_host_MD5_get(URLImpl *url, INK_MD5 *md5)
   ctx.update(":", 1);
 
   // [amc] Why is this <int> and not <in_port_t>?
-  // Especially since it's in_port_t for url_MD5_get.
+  // Especially since it's in_port_t for url_CryptoHash_get.
   int port = url_canonicalize_port(url->m_url_type, url->m_port);
   ctx.update(&port, sizeof(port));
-  ctx.finalize(*md5);
+  ctx.finalize(*hash);
 }
 
 /*-------------------------------------------------------------------------
  * Regression tests
   -------------------------------------------------------------------------*/
 #if TS_HAS_TESTS
-#include "ts/TestBox.h"
+#include "tscore/TestBox.h"
 
 const static struct {
   const char *const text;
@@ -1856,8 +1833,7 @@ REGRESSION_TEST(VALIDATE_HDR_FIELD)(RegressionTest *t, int /* level ATS_UNUSED *
 
   for (auto i : http_validate_hdr_field_test_case) {
     const char *const txt = i.text;
-    ts::ConstBuffer tmp   = ts::ConstBuffer(txt, strlen(txt));
-    box.check(validate_host_name(tmp) == i.valid, "Validation of FQDN (host) header: \"%s\", expected %s, but not", txt,
+    box.check(validate_host_name({txt}) == i.valid, "Validation of FQDN (host) header: \"%s\", expected %s, but not", txt,
               (i.valid ? "true" : "false"));
   }
 }
