@@ -30,18 +30,19 @@
 
  ****************************************************************************/
 
-#ifndef _HTTP_SM_H_
-#define _HTTP_SM_H_
+#pragma once
 
-#include "ts/ink_platform.h"
+#include "tscore/ink_platform.h"
 #include "P_EventSystem.h"
 #include "HttpCacheSM.h"
 #include "HttpTransact.h"
+#include "UrlRewrite.h"
 #include "HttpTunnel.h"
 #include "InkAPIInternal.h"
 #include "../ProxyClientTransaction.h"
 #include "HdrUtils.h"
-#include <ts/MemView.h>
+#include <string_view>
+#include "tscore/History.h"
 //#include "AuthHttpAdapter.h"
 
 /* Enable LAZY_BUF_ALLOC to delay allocation of buffers until they
@@ -127,8 +128,8 @@ private:
 inline bool
 HttpVCTable::is_table_clear() const
 {
-  for (int i = 0; i < vc_table_max_entries; i++) {
-    if (vc_table[i].vc != NULL) {
+  for (const auto &i : vc_table) {
+    if (i.vc != nullptr) {
       return false;
     }
   }
@@ -139,9 +140,8 @@ struct HttpTransformInfo {
   HttpVCTableEntry *entry;
   VConnection *vc;
 
-  HttpTransformInfo() : entry(NULL), vc(NULL) {}
+  HttpTransformInfo() : entry(nullptr), vc(nullptr) {}
 };
-#define HISTORY_SIZE 64
 
 enum {
   HTTP_SM_MAGIC_ALIVE = 0x0000FEED,
@@ -184,10 +184,27 @@ public:
   MIOBuffer *postdata_copy_buffer            = nullptr;
   IOBufferReader *postdata_copy_buffer_start = nullptr;
   IOBufferReader *ua_buffer_reader           = nullptr;
+  bool post_data_buffer_done                 = false;
 
   void clear();
   void init(IOBufferReader *ua_reader);
   void copy_partial_post_data();
+  IOBufferReader *get_post_data_buffer_clone_reader();
+  void
+  set_post_data_buffer_done(bool done)
+  {
+    post_data_buffer_done = done;
+  }
+  bool
+  get_post_data_buffer_done()
+  {
+    return post_data_buffer_done;
+  }
+  bool
+  is_valid()
+  {
+    return postdata_copy_buffer_start != nullptr;
+  }
 
   ~PostDataBuffers();
 };
@@ -253,6 +270,14 @@ public:
   VConnection *do_transform_open();
   VConnection *do_post_transform_open();
 
+  // Called by transact(HttpTransact::is_request_retryable), temperarily.
+  // This function should be remove after #1994 fixed.
+  bool
+  is_post_transform_request()
+  {
+    return t_state.method == HTTP_WKSIDX_POST && post_transform_info.vc;
+  }
+
   // Called from InkAPI.cc which acquires the state machine lock
   //  before calling
   int state_api_callback(int event, void *data);
@@ -274,36 +299,38 @@ public:
   void txn_hook_prepend(TSHttpHookID id, INKContInternal *cont);
   APIHook *txn_hook_get(TSHttpHookID id);
 
-  void add_history_entry(const char *fileline, int event, int reentrant);
-  void add_cache_sm();
   bool is_private();
   bool is_redirect_required();
 
   /// Get the protocol stack for the inbound (client, user agent) connection.
   /// @arg result [out] Array to store the results
   /// @arg n [in] Size of the array @a result.
-  int populate_client_protocol(ts::StringView *result, int n) const;
-  const char *client_protocol_contains(ts::StringView tag_prefix) const;
-  ts::StringView find_proto_string(HTTPVersion version) const;
+  int populate_client_protocol(std::string_view *result, int n) const;
+  const char *client_protocol_contains(std::string_view tag_prefix) const;
+  std::string_view find_proto_string(HTTPVersion version) const;
 
-  int64_t sm_id;
-  unsigned int magic;
+  int64_t sm_id      = -1;
+  unsigned int magic = HTTP_SM_MAGIC_DEAD;
 
   // YTS Team, yamsat Plugin
-  bool enable_redirection; // To check if redirection is enabled
-  char *redirect_url;      // url for force redirect (provide users a functionality to redirect to another url when needed)
-  int redirect_url_len;
-  int redirection_tries;    // To monitor number of redirections
-  int64_t transfered_bytes; // Added to calculate POST data
-  bool post_failed;         // Added to identify post failure
-  bool debug_on;            // Transaction specific debug flag
+  bool enable_redirection = false; // To check if redirection is enabled
+  char *redirect_url    = nullptr; // url for force redirect (provide users a functionality to redirect to another url when needed)
+  int redirect_url_len  = 0;
+  int redirection_tries = 0;        // To monitor number of redirections
+  int64_t transfered_bytes = 0;     // Added to calculate POST data
+  bool post_failed         = false; // Added to identify post failure
+  bool debug_on            = false; // Transaction specific debug flag
 
   // Tunneling request to plugin
-  HttpPluginTunnel_t plugin_tunnel_type;
-  PluginVCCore *plugin_tunnel;
+  HttpPluginTunnel_t plugin_tunnel_type = HTTP_NO_PLUGIN_TUNNEL;
+  PluginVCCore *plugin_tunnel           = nullptr;
 
   HttpTransact::State t_state;
 
+  // This unfortunately can't go into the t_state, beacuse of circular dependencies. We could perhaps refactor
+  // this, with a lot of work, but this is easier for now.
+  UrlRewrite *m_remap = nullptr;
+
   // _postbuf api
   int64_t postbuf_reader_avail();
   int64_t postbuf_buffer_avail();
@@ -311,62 +338,59 @@ public:
   void disable_redirect();
   void postbuf_copy_partial_data();
   void postbuf_init(IOBufferReader *ua_reader);
+  void set_postbuf_done(bool done);
+  bool get_postbuf_done();
+  bool is_postbuf_valid();
+  IOBufferReader *get_postbuf_clone_reader();
 
 protected:
-  int reentrancy_count;
-
-  struct History {
-    const char *fileline;
-    unsigned short event;
-    short reentrancy;
-  };
-  History history[HISTORY_SIZE];
-  int history_pos;
+  int reentrancy_count = 0;
 
   HttpTunnel tunnel;
 
   HttpVCTable vc_table;
 
-  HttpVCTableEntry *ua_entry;
+  HttpVCTableEntry *ua_entry = nullptr;
   void remove_ua_entry();
 
 public:
-  ProxyClientTransaction *ua_session;
-  BackgroundFill_t background_fill;
+  ProxyClientTransaction *ua_txn   = nullptr;
+  BackgroundFill_t background_fill = BACKGROUND_FILL_NONE;
   // AuthHttpAdapter authAdapter;
   void set_http_schedule(Continuation *);
   int get_http_schedule(int event, void *data);
 
-protected:
-  IOBufferReader *ua_buffer_reader;
-  IOBufferReader *ua_raw_buffer_reader;
+  History<HISTORY_DEFAULT_SIZE> history;
 
-  HttpVCTableEntry *server_entry;
-  HttpServerSession *server_session;
+protected:
+  IOBufferReader *ua_buffer_reader     = nullptr;
+  IOBufferReader *ua_raw_buffer_reader = nullptr;
+
+  HttpVCTableEntry *server_entry    = nullptr;
+  HttpServerSession *server_session = nullptr;
 
   /* Because we don't want to take a session from a shared pool if we know that it will be private,
    * but we cannot set it to private until we have an attached server session.
    * So we use this variable to indicate that
    * we should create a new connection and then once we attach the session we'll mark it as private.
    */
-  bool will_be_private_ss;
-  int shared_session_retries;
-  IOBufferReader *server_buffer_reader;
+  bool will_be_private_ss              = false;
+  int shared_session_retries           = 0;
+  IOBufferReader *server_buffer_reader = nullptr;
   void remove_server_entry();
 
   HttpTransformInfo transform_info;
   HttpTransformInfo post_transform_info;
   /// Set if plugin client / user agents are active.
   /// Need primarily for cleanup.
-  bool has_active_plugin_agents;
+  bool has_active_plugin_agents = false;
 
   HttpCacheSM cache_sm;
   HttpCacheSM transform_cache_sm;
-  HttpCacheSM *second_cache_sm;
 
-  HttpSMHandler default_handler;
-  Action *pending_action;
-  Continuation *schedule_cont;
+  HttpSMHandler default_handler = nullptr;
+  Action *pending_action        = nullptr;
+  Continuation *schedule_cont   = nullptr;
 
   HTTPParser http_parser;
   void start_sub_sm();
@@ -391,11 +415,9 @@ protected:
   int state_hostdb_reverse_lookup(int event, void *data);
   int state_mark_os_down(int event, void *data);
   int state_handle_stat_page(int event, void *data);
-  int state_icp_lookup(int event, void *data);
   int state_auth_callback(int event, void *data);
   int state_add_to_list(int event, void *data);
   int state_remove_from_list(int event, void *data);
-  int state_congestion_control_lookup(int event, void *data);
 
   // Y! ebalsa: remap handlers
   int state_remap_request(int event, void *data);
@@ -443,7 +465,6 @@ protected:
   void do_cache_prepare_update();
   void do_cache_prepare_action(HttpCacheSM *c_sm, CacheHTTPInfo *object_read_info, bool retry, bool allow_multiple = false);
   void do_cache_delete_all_alts(Continuation *cont);
-  void do_icp_lookup();
   void do_auth_callout();
   void do_api_callout();
   void do_api_callout_internal();
@@ -451,7 +472,7 @@ protected:
   void redirect_request(const char *redirect_url, const int redirect_len);
   void do_drain_request_body();
 
-  bool do_congestion_control_lookup();
+  void wait_for_full_body();
 
   virtual void handle_api_return();
   void handle_server_setup_error(int event, void *data);
@@ -485,15 +506,15 @@ protected:
   void perform_cache_write_action();
   void perform_transform_cache_write_action();
   void perform_nca_cache_action();
-  void setup_blind_tunnel(bool send_response_hdr);
+  void setup_blind_tunnel(bool send_response_hdr, IOBufferReader *initial = nullptr);
   HttpTunnelProducer *setup_server_transfer_to_transform();
   HttpTunnelProducer *setup_transfer_from_transform();
   HttpTunnelProducer *setup_cache_transfer_to_transform();
   HttpTunnelProducer *setup_transfer_from_transform_to_cache_only();
   void setup_plugin_agents(HttpTunnelProducer *p);
 
-  HttpTransact::StateMachineAction_t last_action;
-  int (HttpSM::*m_last_state)(int event, void *data);
+  HttpTransact::StateMachineAction_t last_action     = HttpTransact::SM_ACTION_UNDEFINED;
+  int (HttpSM::*m_last_state)(int event, void *data) = nullptr;
   virtual void set_next_state();
   void call_transact_and_set_next_state(TransactEntryFunc_t f);
 
@@ -504,52 +525,55 @@ protected:
   int64_t server_transfer_init(MIOBuffer *buf, int hdr_size);
 
 public:
+  // TODO:  Now that bodies can be empty, should the body counters be set to -1 ? TS-2213
   // Stats & Logging Info
-  int client_request_hdr_bytes;
-  int64_t client_request_body_bytes;
-  int server_request_hdr_bytes;
-  int64_t server_request_body_bytes;
-  int server_response_hdr_bytes;
-  int64_t server_response_body_bytes;
-  int client_response_hdr_bytes;
-  int64_t client_response_body_bytes;
-  int cache_response_hdr_bytes;
-  int64_t cache_response_body_bytes;
-  int pushed_response_hdr_bytes;
-  int64_t pushed_response_body_bytes;
-  bool client_tcp_reused;
+  int client_request_hdr_bytes       = 0;
+  int64_t client_request_body_bytes  = 0;
+  int server_request_hdr_bytes       = 0;
+  int64_t server_request_body_bytes  = 0;
+  int server_response_hdr_bytes      = 0;
+  int64_t server_response_body_bytes = 0;
+  int client_response_hdr_bytes      = 0;
+  int64_t client_response_body_bytes = 0;
+  int cache_response_hdr_bytes       = 0;
+  int64_t cache_response_body_bytes  = 0;
+  int pushed_response_hdr_bytes      = 0;
+  int64_t pushed_response_body_bytes = 0;
+  bool client_tcp_reused             = false;
   // Info about client's SSL connection.
-  bool client_ssl_reused;
-  bool client_connection_is_ssl;
-  const char *client_protocol;
-  const char *client_sec_protocol;
-  const char *client_cipher_suite;
-  int server_transact_count;
-  bool server_connection_is_ssl;
+  bool client_ssl_reused          = false;
+  bool client_connection_is_ssl   = false;
+  const char *client_protocol     = "-";
+  const char *client_sec_protocol = "-";
+  const char *client_cipher_suite = "-";
+  int server_transact_count       = 0;
+  bool server_connection_is_ssl   = false;
+  bool is_waiting_for_full_body   = false;
+  bool is_using_post_buffer       = false;
 
   TransactionMilestones milestones;
-  ink_hrtime api_timer;
+  ink_hrtime api_timer = 0;
   // The next two enable plugins to tag the state machine for
   // the purposes of logging so the instances can be correlated
   // with the source plugin.
-  const char *plugin_tag;
-  int64_t plugin_id;
+  const char *plugin_tag = nullptr;
+  int64_t plugin_id      = 0;
 
   // hooks_set records whether there are any hooks relevant
   //  to this transaction.  Used to avoid costly calls
   //  do_api_callout_internal()
-  bool hooks_set;
+  bool hooks_set = false;
 
 protected:
-  TSHttpHookID cur_hook_id;
-  APIHook *cur_hook;
+  TSHttpHookID cur_hook_id = TS_HTTP_LAST_HOOK;
+  APIHook *cur_hook        = nullptr;
 
   //
   // Continuation time keeper
-  int64_t prev_hook_start_time;
+  int64_t prev_hook_start_time = 0;
 
-  int cur_hooks;
-  HttpApiState_t callout_state;
+  int cur_hooks                = 0;
+  HttpApiState_t callout_state = HTTP_API_NO_CALLOUT;
 
   // api_hooks must not be changed directly
   //  Use txn_hook_{ap,pre}pend so hooks_set is
@@ -559,9 +583,9 @@ protected:
   // The terminate flag is set by handlers and checked by the
   //   main handler who will terminate the state machine
   //   when the flag is set
-  bool terminate_sm;
-  bool kill_this_async_done;
-  bool parse_range_done;
+  bool terminate_sm         = false;
+  bool kill_this_async_done = false;
+  bool parse_range_done     = false;
   virtual int kill_this_async_hook(int event, void *data);
   void kill_this();
   void update_stats();
@@ -580,8 +604,21 @@ public:
     return terminate_sm;
   }
 
+  int
+  client_connection_id() const
+  {
+    return _client_connection_id;
+  }
+
+  int
+  client_transaction_id() const
+  {
+    return _client_transaction_id;
+  }
+
 private:
   PostDataBuffers _postbuf;
+  int _client_connection_id = -1, _client_transaction_id = -1;
 };
 
 // Function to get the cache_sm object - YTS Team, yamsat
@@ -609,7 +646,7 @@ inline void
 HttpSM::remove_ua_entry()
 {
   vc_table.remove_entry(ua_entry);
-  ua_entry = NULL;
+  ua_entry = nullptr;
 }
 
 inline void
@@ -617,7 +654,7 @@ HttpSM::remove_server_entry()
 {
   if (server_entry) {
     vc_table.remove_entry(server_entry);
-    server_entry = NULL;
+    server_entry = nullptr;
   }
 }
 
@@ -631,15 +668,6 @@ HttpSM::write_response_header_into_buffer(HTTPHdr *h, MIOBuffer *b)
   }
 }
 
-inline void
-HttpSM::add_history_entry(const char *fileline, int event, int reentrant)
-{
-  int pos                 = history_pos++ % HISTORY_SIZE;
-  history[pos].fileline   = fileline;
-  history[pos].event      = (unsigned short)event;
-  history[pos].reentrancy = (short)reentrant;
-}
-
 inline int
 HttpSM::find_server_buffer_size()
 {
@@ -650,14 +678,14 @@ inline void
 HttpSM::txn_hook_append(TSHttpHookID id, INKContInternal *cont)
 {
   api_hooks.append(id, cont);
-  hooks_set = 1;
+  hooks_set = true;
 }
 
 inline void
 HttpSM::txn_hook_prepend(TSHttpHookID id, INKContInternal *cont)
 {
   api_hooks.prepend(id, cont);
-  hooks_set = 1;
+  hooks_set = true;
 }
 
 inline APIHook *
@@ -666,27 +694,10 @@ HttpSM::txn_hook_get(TSHttpHookID id)
   return api_hooks.get(id);
 }
 
-inline void
-HttpSM::add_cache_sm()
-{
-  if (second_cache_sm == NULL) {
-    second_cache_sm = new HttpCacheSM;
-    second_cache_sm->init(this, mutex);
-    if (t_state.cache_info.object_read != NULL) {
-      second_cache_sm->cache_read_vc        = cache_sm.cache_read_vc;
-      cache_sm.cache_read_vc                = NULL;
-      second_cache_sm->read_locked          = cache_sm.read_locked;
-      t_state.cache_info.second_object_read = t_state.cache_info.object_read;
-      t_state.cache_info.object_read        = NULL;
-    }
-  }
-}
-
 inline bool
 HttpSM::is_transparent_passthrough_allowed()
 {
-  return (t_state.client_info.is_transparent && ua_session->is_transparent_passthrough_allowed() &&
-          ua_session->get_transact_count() == 1);
+  return (t_state.client_info.is_transparent && ua_txn->is_transparent_passthrough_allowed() && ua_txn->get_transact_count() == 1);
 }
 
 inline int64_t
@@ -726,4 +737,26 @@ HttpSM::postbuf_init(IOBufferReader *ua_reader)
   this->_postbuf.init(ua_reader);
 }
 
-#endif
+inline void
+HttpSM::set_postbuf_done(bool done)
+{
+  this->_postbuf.set_post_data_buffer_done(done);
+}
+
+inline bool
+HttpSM::get_postbuf_done()
+{
+  return this->_postbuf.get_post_data_buffer_done();
+}
+
+inline bool
+HttpSM::is_postbuf_valid()
+{
+  return this->_postbuf.is_valid();
+}
+
+inline IOBufferReader *
+HttpSM::get_postbuf_clone_reader()
+{
+  return this->_postbuf.get_post_data_buffer_clone_reader();
+}
